#NETWORK POST PROCESSING
#It contains functions that computes additional informations (e.g. flow matrices, biomass, 
#bioconcentration factors, bioaccumulation factors, etc) from the estimated flows

#Trophic model post-processing
#it takes either a LIM model to estimate, or an already computed solution. 
#and returns a set of flow matrices (full, without externals, with no flows to detritus), 
#the estimated biomasses, imports and exports vectors.
estBiomassNet<- function(LIMmodel,
                         exactEqs=NULL,#vector of the indices of the exact equations
                         method="Lsei", #LIM solution method
                         solution, #already computed solution
                         inputParams, #table of input parameters (as parsed from the input file)
                         respNams=c("CO2"),#respiration group names
                         impNams=c("IMP"),#import group names
                         expNams=c("EXP"),#export group names
                         fishNams=c("LAND","DISC"),#fishing group names (only the external)
                         cost=NULL#cost function
){
  
  if(!is.null(LIMmodel)){
    if(method=="Lsei" && is.null(exactEqs)){
      SSA <- Lsei(LIMmodel)
    } else if(method=="Lsei"){
      SSA <- Lsei(LIMmodel, exact = exactEqs)
    } else if(method=="Ldei"){
      SSA <- Ldei(LIMmodel)
    } else if(method=="Linp"){
      if(is.null(cost))
        SSA <- Linp(LIMmodel)
      else SSA <- Linp(LIMmodel,cost)
      rnames<-rownames(t(SSA$X))
      SSA$X<-as.vector(t(SSA$X))
      names(SSA$X)<-rnames
    }
  }
  else if(!is.null(solution)){
    SSA<-list(X=solution)
  }
  else stop("Both LIM model and solution are null")
  
  #input checking
  #extNames1<-toupper(c(respNam,impName,expNam))
  respNams<-toupper(respNams)
  impNams<-toupper(impNams)
  expNams<-toupper(expNams)
  detNams<-inputParams$Group[inputParams$Det==1]
  ppNams<-inputParams$Group[inputParams$PP==1]
  discardNams<-inputParams$Group[inputParams$Discard==1]
  checkExt<-c(respNams,impNams,expNams,fishNams)
  
  extNames<-toupper(as.vector(LIMmodel$Externals$name))
  compNames<-as.vector(LIMmodel$Components$name)
  
  if(!checkExt %in% extNames || !inputParams$Group %in% compNames)
    stop("External or compartment names do not match")
  
  #function for zeroing flow to detritus and to discards
  zeroFD<-function(flowM){
    flwMatrix_noFD<-flowM
    flwMatrix_noFD[,detNams]<-0
    #flwMatrix_noFD[,discardNams]<-0
    return(flwMatrix_noFD)
  }
  
  #remove externals
  removeExt<-function(flowM){
    flwMatrix_noext<-flowM[!rownames(flowM) %in% extNames,
                           !colnames(flowM) %in% extNames]
    return(flwMatrix_noext)
  }
  
  #remove discards
  removeDisc<-function(flowM){
    flwMatrix_noext<-flowM[!rownames(flowM) %in% discardNams,
                           !colnames(flowM) %in% discardNams]
    return(flwMatrix_noext)
  }
  
  flwMatrix<-matrix(0,nrow=LIMmodel$NComponents+LIMmodel$NExternal,
                    ncol=LIMmodel$NComponents+LIMmodel$NExternal)
  rownames(flwMatrix)<-c(extNames,compNames)
  colnames(flwMatrix)<-c(extNames,compNames)
  
  #set up matrix
  for(i in 1:length(SSA$X)){
    currName<-toupper(names(SSA$X)[i])
    src<-unlist(strsplit(currName,"->"))[1]
    trg<-unlist(strsplit(currName,"->"))[2]
    flwMatrix[src,trg]<-as.vector(SSA$X[i])
  }
  flwMatrix_noext<-removeExt(flwMatrix)
  
  #compute biomasses, works with exact consumption values, or with exact biomasses when cons. is not available
  biomasses<-matrix(flwMatrix_noext[,1])
  rownames(biomasses)<-rownames(flwMatrix_noext)
  #colnames(biomasses)<-"Biomass"
  LIMparams<-LIMmodel$Parameters
  LIMparams$name<-toupper(LIMparams$name)
  
  for(i in 1:ncol(flwMatrix_noext)){
    nam<-colnames(flwMatrix_noext)[i]
    ingested<-sum(flwMatrix_noext[,i])
    conspt<-LIMparams[LIMparams$name==paste("C_",nam,sep=""),"val"]
    
    if(length(conspt)!=0)
      biomasses[i]<-ingested/conspt
    else
      biomasses[i]<-LIMparams[LIMparams$name==paste("B_",nam,sep=""),"val"]
  }
  
  #respiration vector
  resp<-flwMatrix[!rownames(flwMatrix) %in% extNames,colnames(flwMatrix) %in% respNams]
  #import vector
  imp<-flwMatrix[rownames(flwMatrix) %in% impNams,!colnames(flwMatrix) %in% extNames]
  #export vector
  exp<-flwMatrix[!rownames(flwMatrix) %in% extNames,colnames(flwMatrix) %in% c(expNams,fishNams)]
  #fishing vector
  fishing<-flwMatrix[!rownames(flwMatrix) %in% extNames,colnames(flwMatrix) %in% c(fishNams,discardNams)]
  
  #living vector
  living<-resp
  living[1:length(living)]<-TRUE
  #both detritus and discards are considered non-living
  living[detNams]<-FALSE
  living[discardNams]<-FALSE
  
  
  #compute rates of change
  changeRates<-matrix(flwMatrix[,1])
  rownames(changeRates)<-rownames(flwMatrix)
  colnames(changeRates)<-"Change rate"
  for(i in 1:nrow(changeRates))
    changeRates[i]<-sum(flwMatrix[,i])-sum(flwMatrix[i,])
  
  #compute diet composition matrix
  dietComp<-zeroFD(flwMatrix_noext)
  for(i in 1:ncol(dietComp)){
    inSum<-sum(dietComp[,i])
    if(inSum>0)
      for(j in 1:nrow(dietComp))
        dietComp[j,i]<-dietComp[j,i]/inSum
  }
  
  return(list(solution=SSA,flowM=flwMatrix,flowM_no_FD=zeroFD(flwMatrix),flowM_no_extDiscFD=zeroFD(removeDisc(flwMatrix)),
              flowM_no_ext=flwMatrix_noext, flowM_no_extFD=zeroFD(flwMatrix_noext),
              biomass=biomasses,respiration=resp,imports=imp,exports=exp,fishing=fishing,living=living,
              changeR=changeRates,dietComposition=dietComp))
}

#Contaminant model post-processing
#it takes a computed solution of the bioaccumulation LIM,
#and returns a set of flow matrices (full, without externals, with no flows to detritus), 
#the estimated concentrations, and contaminant flows, imports and exports vectors,
#uptake from water vectors, BCF, BAF and BMF.
estContaminantNet<- function(solution,LIMmodelbiomass,res_biomass,
                             inputParams,
                             waterNam="WATER",#water group name (without initial 'C_')
                             respNams=c("CO2"),#respiration group names
                             impNams=c("IMP"),#import group names
                             expNams=c("EXP"),#export group names
                             fishNams=c("LAND","DISC"),#fishing group names (only the external)
                             LIPID_INST=0.0053, LIPID_KINE=0.02){
  
  relativeFlows<-inputParams$relativeFlows
  inputParams<-inputParams$inputTable
  discardNams<-inputParams$Group[inputParams$Discard==1]
  
  waterGrpNam=paste("C_",waterNam,sep="")
  respNams<-toupper(respNams)
  impNams<-toupper(impNams)
  expNams<-toupper(expNams)
  detNams<-inputParams$Group[inputParams$Det==1]
  ppNams<-inputParams$Group[inputParams$PP==1]
  instNams<-inputParams$Group[inputParams$InstEq==1]
  checkExt<-c(respNams,impNams,expNams,fishNams,waterNam)
  
  extNames<-c(toupper(as.vector(LIMmodelbiomass$Externals$name)),waterNam)
  compNames<-as.vector(LIMmodelbiomass$Components$name)
  
  if(!checkExt %in% extNames || !inputParams$Group %in% compNames)
    stop("External or compartment names do not match")
  
  #function for zeroing flow to detritus
  zeroFD<-function(flowM){
    flwMatrix_noFD<-flowM
    flwMatrix_noFD[,detNams]<-0
    #flwMatrix_noFD[,discardNams]<-0
    return(flwMatrix_noFD)
  }
  
  #function for zeroing flow to inst compartments
  zeroInst<-function(flowM){
    flwMatrix_noInst<-flowM
    flwMatrix_noInst[,instNams]<-0
    return(flwMatrix_noInst)
  }
  
  #remove externals
  removeExt<-function(flowM){
    flwMatrix_noext<-flowM[!rownames(flowM) %in% checkExt,
                           !colnames(flowM) %in% checkExt]
    return(flwMatrix_noext)
  }
  
  #remove discards
  removeDisc<-function(flowM){
    flwMatrix_noext<-flowM[!rownames(flowM) %in% discardNams,
                           !colnames(flowM) %in% discardNams]
    return(flwMatrix_noext)
  }
  
  #extract bioconcentrations
  bioconcs<-data.frame(Conc=solution[rownames(solution) %in% paste("C_",compNames,sep=""),])
  rownames(bioconcs)<-substring(rownames(solution)[rownames(solution) %in% paste("C_",compNames,sep="")],3)
  #mg/L (consistent with PCB concentration on species ng/g)
  cwater<-solution[waterGrpNam,]
  #extract uptake rates from water per unit of water concentration
  uptakeWrate<-data.frame(u_w=solution[rownames(solution) %in% paste("W_",compNames,sep=""),])/cwater
  rownames(uptakeWrate)<-substring(rownames(solution)[rownames(solution) %in% paste("W_",compNames,sep="")],3)
  colnames(uptakeWrate)<-"Uptake Rate"
  
  #compute flow matrix
  flwMatrix<-matrix(0,nrow=length(extNames)+length(compNames),
                    ncol=length(extNames)+length(compNames))
  rownames(flwMatrix)<-c(extNames,compNames)
  colnames(flwMatrix)<-c(extNames,compNames)
  
  for(i in 1:nrow(res_biomass$flowM)){
    nameI<-rownames(res_biomass$flowM)[i]
    for(j in 1:ncol(res_biomass$flowM)){
      nameJ<-colnames(res_biomass$flowM)[j]
      if(nameI %in% impNams && nameJ %in% compNames){
        if(relativeFlows)
          flwMatrix[nameI,nameJ]<-res_biomass$flowM[i,j]/res_biomass$biomass[nameJ,]*bioconcs[nameJ,]
        else flwMatrix[nameI,nameJ]<-res_biomass$flowM[i,j]*bioconcs[nameJ,]
      }
      else if(nameI %in% compNames){
        if(relativeFlows)
          flwMatrix[nameI,nameJ]<-res_biomass$flowM[i,j]/res_biomass$biomass[nameI,]*bioconcs[nameI,]
        else flwMatrix[nameI,nameJ]<-res_biomass$flowM[i,j]*bioconcs[nameI,]
      }
    }
  }
  #add inflows from water
  for(j in 1:ncol(flwMatrix)){
    nameJ<-colnames(flwMatrix)[j]
    if(!nameJ %in% instNams && nameJ %in% compNames)
      flwMatrix[waterNam,j]<-uptakeWrate[nameJ,]*cwater
  }
  
  flwMatrix_noext<-removeExt(flwMatrix)
  
  #respiration vector
  resp<-flwMatrix[!rownames(flwMatrix) %in% checkExt,colnames(flwMatrix) %in% respNams]
  #import vector
  imp<-flwMatrix[rownames(flwMatrix) %in% impNams,!colnames(flwMatrix) %in% checkExt]
  #export vector
  exp<-flwMatrix[!rownames(flwMatrix) %in% checkExt,colnames(flwMatrix) %in% c(fishNams,expNams)]
  #fishing vector
  fishing<-flwMatrix[!rownames(flwMatrix) %in% checkExt,colnames(flwMatrix) %in% c(fishNams,discardNams)]
  
  #living vector
  living<-resp
  living[1:length(living)]<-TRUE
  #both detritus and discards are considered non-living
  living[detNams]<-FALSE
  living[discardNams]<-FALSE
  
  #compute rates of change 
  changeRates<-matrix(flwMatrix[,1])
  rownames(changeRates)<-rownames(flwMatrix)
  colnames(changeRates)<-"Change rate"
  for(i in 1:nrow(changeRates))
    changeRates[i]<-sum(flwMatrix[,i])-sum(flwMatrix[i,])
  
  #compute uptake from diet matrix
  dietComp<-zeroFD(flwMatrix_noext)
  for(i in 1:ncol(dietComp)){
    inSum<-sum(dietComp[,i])
    if(inSum>0)
      for(j in 1:nrow(dietComp))
        dietComp[j,i]<-dietComp[j,i]/inSum
  }
  
  #set up vector of lipid fractions replacing NAs with default values
  lipid<-data.frame(LipidFrac=inputParams$LipidFrac)
  rownames(lipid)<-inputParams$Group
  for(i in 1:nrow(lipid)){
    if(rownames(lipid)[i] %in% instNams && is.na(lipid[i,]))
      lipid[i,]<-LIPID_INST
    else if(is.na(lipid[i,]))
      lipid[i,]<-LIPID_KINE
  }
  colnames(lipid)<-"LIPID"
  
  #bioaccumulation factor, KINE species (BAF=C_i/C_water)
  baf<-data.frame(BAF=bioconcs[!rownames(bioconcs) %in% instNams,])
  rownames(baf)<-rownames(bioconcs)[!rownames(bioconcs) %in% instNams]
  baf<-baf/cwater
  #lipid normalized baf
  baf_l<-baf/lipid[!rownames(lipid) %in% instNams,]
  
  #bioconcentration factor, INST species (BCF=C_i/C_water)
  bcf<-data.frame(BCF=bioconcs[rownames(bioconcs) %in% instNams,])
  rownames(bcf)<-rownames(bioconcs)[rownames(bioconcs) %in% instNams]
  bcf<-bcf/cwater
  #lipid normalized baf
  bcf_l<-bcf/lipid[rownames(lipid) %in% instNams,]
  
  #biomagnification factor 
  #(weigthed (biomass diet composition matrix) ratio of a species' concentration and its prey's concentrations)
  bmf<-bioconcs
  colnames(bmf)<-"BMF"
  diet<-res_biomass$dietComposition
  for(i in 1:nrow(bioconcs)){
    if(sum(diet[,rownames(bmf)[i]]>0)){
      preyconc<-0
      for(j in 1:nrow(diet))
        preyconc<-preyconc+diet[j,i]*bioconcs[rownames(diet)[j],]
      bmf[i,]<-bioconcs[i,]/preyconc
    }
    else 
      bmf[i,]<-NA
  }
  #lipid normalized bmf
  bmf_l<-bmf
  for(i in 1:nrow(bioconcs)){
    if(sum(diet[,rownames(bmf_l)[i]]>0)){
      preyconc<-0
      for(j in 1:nrow(diet))
        preyconc<-preyconc+diet[j,i]*bioconcs[rownames(diet)[j],]/lipid[rownames(diet)[j],]
      bmf_l[i,]<-bioconcs[i,]/lipid[i,]/preyconc
    }
    else bmf_l[i,]<-NA
  }
  
  return(list(solution=solution,flowM=flwMatrix,flowM_no_FD=zeroFD(flwMatrix),
              flowM_no_extDiscFD=zeroFD(removeDisc(flwMatrix)),
              flowM_no_ext=flwMatrix_noext, flowM_no_extFD=zeroFD(flwMatrix_noext),
              concs=bioconcs, conc_water=cwater, u_water=uptakeWrate, dietcomposition=dietComp,
              bcf=bcf, bcf_l=bcf_l, baf=baf, baf_l=baf_l, bmf=bmf, bmf_l=bmf_l,
              respiration=resp,imports=imp,exports=exp,living=living,lipidFrac=lipid,fishing=fishing,
              changeR=changeRates))
}

writeFlowsAsReactions<-function(res_cont,inputParams,outdir,
                                waterNam="WATER",#water group name (without initial 'C_')
                                respNams=c("CO2"),#respiration group names
                                impNams=c("IMP"),#import group names
                                expNams=c("EXP"),#export group names
                                fishNams=c("LAND","DISC"),#fishing group names (only the external)
                                pcbsMets=c("4chlbiph[c]","Biph[c]","carbz[c]"),#names of metabolites representing PCBs
                                commonGroup="commonGroup[c]",#name of the common group in the 3rd scenario
                                upperBound=1e6 #default upper bounds for reaction rates
){
  
  
  if(outdir!=""&&substr(outdir,nchar(outdir),nchar(outdir)+1)!="/")
    outdir<-paste(outdir,"/",sep="")
  
  waterGrpNam=paste("C_",waterNam,sep="")
  respNams<-toupper(respNams)
  impNams<-toupper(impNams)
  expNams<-toupper(expNams)
  detNams<-inputParams$Group[inputParams$Det==1]
  ppNams<-inputParams$Group[inputParams$PP==1]
  instNams<-inputParams$Group[inputParams$InstEq==1]
  discardName<-inputParams$Group[inputParams$Discard==1]
  imps<-c(impNams,waterNam)
  exps<-c(respNams,expNams,fishNams)
  exts<-c(respNams,impNams,expNams,fishNams,waterNam,instNams)
  
  reactions <-c()
  ub <- c()
  lb <- c()
  ids <- c()
  flowM<-res_cont$flowM
  #zero self-flows
  for(i in 1:nrow(flowM)){
    flowM[i,i]<-0
  }
  for(i in 1:nrow(flowM)){
    sourceN <- rownames(flowM)[i]
    if(sourceN %in% exts){
      #external reaction handling
      outFlws<- sum(flowM[i,])
      inflows<- sum(flowM[,i])
      reactId <- paste("r_FW_",i,"_Extern",sep="")
      ids <- c(ids,reactId)
      if(sourceN %in% imps){
        reactions<-c(reactions,paste(sourceN,"[e]", " -> ",sep=""))
        ub <- c(ub,-outFlws)
        lb <- c(lb,-outFlws)
      }
      else if(sourceN %in% exps){
        reactions<-c(reactions,paste(sourceN,"[e]"," -> ", sep=""))
        ub <- c(ub,upperBound)
        lb <- c(lb,0)
      }
      else if (outFlws-inflows>0) {
        reactions<-c(reactions,paste(sourceN,"[e]", " -> ",sep=""))
        ub <- c(ub,0)
        lb <- c(lb,inflows-outFlws)
      }
      else {
        reactions<-c(reactions,paste(sourceN,"[e]"," -> ", sep=""))
        ub <- c(ub,inflows-outFlws)
        lb <- c(lb,0)
      }
      sourceN <- paste(sourceN, "[e]", sep="")
    }
    else {sourceN <- paste(sourceN, "[c]", sep="")
    }
    
    
    for(j in 1:ncol(flowM)){
      if(flowM[i,j]>0){
        targetN <- colnames(flowM)[j]
        if(targetN %in% exts)
          targetN <- paste(targetN, "[e]", sep="")
        else targetN <- paste(targetN, "[c]", sep="")
        reactId <- paste("r_FW_",i,"_",j,sep="")
        reactions<-c(reactions,paste(sourceN," -> ",targetN, sep=""))
        ub <- c(ub,flowM[i,j])
        lb <- c(lb,0)
        ids <- c(ids,reactId)
      }
    }
  }
  
  #connection with bacteria
  for(i in 1:length(pcbsMets)){
    reactId <- paste("r_FW_",i,"_connect",sep="")
    reactions<-c(reactions,paste(commonGroup," -> ",pcbsMets[i], sep=""))
    ub <- c(ub,upperBound)
    lb <- c(lb,0)
    ids <- c(ids,reactId)
  }
  
  #reaction for directing detritus into common DETRITI group
  ids <- c(ids,"r_FW_detritus_common")
  ub <- c(ub,upperBound)
  lb <- c(lb,0)
  reactions<-c(reactions,paste(detNams[1],"[e] -> DETRITI[c]",sep=""))
  #reaction for connecting water to the common group
  ids <- c(ids,"r_FW_water_common")
  ub <- c(ub,upperBound)
  lb <- c(lb,0)
  reactions<-c(reactions,paste(waterNam,"[e] -> ",commonGroup, sep=""))
  #reaction for directing discard into common DETRITI group
  ids <- c(ids,"r_FW_discard_common")
  ub <- c(ub,upperBound)
  lb <- c(lb,0)
  reactions<-c(reactions,paste(discardName[1],"[e] -> DETRITI[c]",sep=""))
  #reaction for connecting detriti to the common group
  ids <- c(ids,"r_FW_detriti_common")
  ub <- c(ub,upperBound)
  lb <- c(lb,0)
  reactions<-c(reactions,paste("DETRITI[c] -> ",commonGroup, sep=""))
  tableToWrite <- data.frame(reactionId=ids,reaction=reactions,lowerBound=lb,upperBound=ub)
  write.table(tableToWrite,file=paste(outdir,"encoded_FW.csv",sep=""),sep=",")
}

#load food web from Matlab vector
parseFlowMatrixFromReactionList<-function(path,numCompartments,names,
                                          biomassFlwMatrix,res_contaminant,res_TL,
                                          detNams=c("DETRITUS"),
                                          waterNam="WATER",#water group name (without initial 'C_')
                                          respNams=c("CO2"),#respiration group names
                                          impNams=c("IMP"),#import group names
                                          expNams=c("EXP"),#export group names
                                          fishNams=c("LAND","DISC")){
  
  flowMatrix<-matrix(0,nrow=numCompartments,ncol=numCompartments)
  colnames(flowMatrix)<-names
  rownames(flowMatrix)<-names
  bioremFlws<-read.csv(file=path,header=FALSE)
  for(i in 1:nrow(bioremFlws)){
    splits<-unlist(strsplit(as.character(bioremFlws[i,1]),'_'))
    groupI<-as.integer(splits[length(splits)-1])
    groupJ<-as.integer(splits[length(splits)])
    if(!(is.na(groupJ)&&is.na(groupI))){
      flowMatrix[groupI,groupJ]<-bioremFlws[i,2]
    }
  }
  #compute back bioconcentrations c_i=sum_j c_ij / sum_j f_ij
  bioconcs <- res_contaminant$concs
  for(i in 1:nrow(bioconcs)){
    idx1=(rownames(flowMatrix)==rownames(bioconcs)[i])
    idx2=(rownames(biomassFlwMatrix)==rownames(bioconcs)[i])
    bioconcs$Conc[i]<-sum(flowMatrix[idx1,])/sum(biomassFlwMatrix[idx2,])
  }
  
  #post-process res_contaminant object
  #we just need to have consistent bioconcs, 
  #dietCompmatrix, flowM_no_ext, imports, exports, respiration, living
  checkExt<-c(respNams,impNams,expNams,fishNams,waterNam)
  res_contaminant1<-res_contaminant
  res_contaminant1$concs<-bioconcs
  res_contaminant1$flowM<-flowMatrix
  res_contaminant1$flowM_no_ext<-flowMatrix[!rownames(flowMatrix) %in% checkExt,
                                            !colnames(flowMatrix) %in% checkExt]
  
  res_contaminant1$respiration<-flowMatrix[!rownames(flowMatrix) %in% checkExt,colnames(flowMatrix) %in% respNams]
  res_contaminant1$imports<-flowMatrix[rownames(flowMatrix) %in% impNams,!colnames(flowMatrix) %in% checkExt]
  res_contaminant1$exports<-flowMatrix[!rownames(flowMatrix) %in% checkExt,colnames(flowMatrix) %in% c(fishNams,expNams)]
  
  dietComp<-res_contaminant1$flowM_no_ext
  dietComp[,detNams]<-0
  for(i in 1:ncol(dietComp)){
    inSum<-sum(dietComp[,i])
    if(inSum>0)
      for(j in 1:nrow(dietComp))
        dietComp[j,i]<-dietComp[j,i]/inSum
  }
  res_contaminant1$dietcomposition<-dietComp
  
  res_contaminant1n<-res_contaminant1
  rownames(res_contaminant1n$concs)=1:nrow(res_contaminant1n$concs)
  res_contaminantn<-res_contaminant
  rownames(res_contaminantn$concs)=1:nrow(res_contaminantn$concs)

  mti_analysis_cont<-MTI(flowM=res_contaminant1$flowM_no_ext,
                         inputM=res_contaminant1$imports, 
                         expM=rowSums(res_contaminant1$exports), 
                         respM=res_contaminant1$respiration, 
                         liveM=res_contaminant1$living, 
                         storM=as.vector(res_contaminant1$concs))
  
  fbc<-keystoneness(enaRmodel=mti_analysis_cont$enaRmodel,storM=res_contaminant1$concs,
                    flowM=res_contaminant1$flowM_no_ext)$flowB
  
  flowMatrix<-cbind(DISC=0,flowMatrix)
  flowMatrix<-rbind(DISC=0,flowMatrix)
  ld<- GenInd(Flow=flowMatrix,Import=c("IMP","WATER","DISC"),Export=c("EXP","CO2","LAND"))$LD
  
  return(list(contflowMatrix=flowMatrix,bioconcs=bioconcs,fbc=fbc,dietMatrix=dietComp,ld=ld))
}


#write to files the result of the trophic network post-processing
writeBiomassEstimationToFile<-function(res,outdir){
  if(outdir!=""&&substr(outdir,nchar(outdir),nchar(outdir)+1)!="/")
    outdir<-paste(outdir,"/",sep="")
  
  write.table(res$solution,file=paste(outdir,"solution.csv",sep=""),sep=",")
  write.table(res$flowM,file=paste(outdir,"flowMatrix.csv",sep=""),sep=",")
  write.table(res$flowM_no_FD,file=paste(outdir,"flowMatrix_noFD.csv",sep=""),sep=",")
  write.table(res$flowM_no_ext,file=paste(outdir,"flowMatrix_noExt.csv",sep=""),sep=",")
  write.table(res$flowM_no_extFD,file=paste(outdir,"flowMatrix_noFD_noExt.csv",sep=""),sep=",")
  write.table(res$biomass,file=paste(outdir,"biomass.csv",sep=""),sep=",")
  write.table(res$respiration,file=paste(outdir,"respiration.csv",sep=""),sep=",")
  write.table(res$imports,file=paste(outdir,"imports.csv",sep=""),sep=",")
  write.table(res$exports,file=paste(outdir,"exports.csv",sep=""),sep=",")
  write.table(res$living,file=paste(outdir,"living.csv",sep=""),sep=",")
  write.table(res$fishing,file=paste(outdir,"fishing.csv",sep=""),sep=",")
  write.table(res$changeR,file=paste(outdir,"changeRates.csv",sep=""),sep=",")
  write.table(res$dietComposition,file=paste(outdir,"dietComposition.csv",sep=""),sep=",")
}

#write to files the result of the bioaccumulation network post-processing
writeContaminantEstimationToFile<-function(res,outdir){
  if(outdir!=""&&substr(outdir,nchar(outdir),nchar(outdir)+1)!="/")
    outdir<-paste(outdir,"/",sep="")
  
  write.table(res$solution,file=paste(outdir,"solution.csv",sep=""),sep=",")
  write.table(res$flowM,file=paste(outdir,"flowMatrix.csv",sep=""),sep=",")
  write.table(res$flowM_no_FD,file=paste(outdir,"flowMatrix_noFD.csv",sep=""),sep=",")
  write.table(res$flowM_no_ext,file=paste(outdir,"flowMatrix_noExt.csv",sep=""),sep=",")
  write.table(res$flowM_no_extFD,file=paste(outdir,"flowMatrix_noFD_noExt.csv",sep=""),sep=",")
  write.table(res$concs,file=paste(outdir,"concentrations.csv",sep=""),sep=",")
  write.table(res$conc_water,file=paste(outdir,"water_concentration.csv",sep=""),sep=",")
  write.table(res$u_water,file=paste(outdir,"water_uptake.csv",sep=""),sep=",")
  write.table(res$respiration,file=paste(outdir,"respiration.csv",sep=""),sep=",")
  write.table(res$imports,file=paste(outdir,"imports.csv",sep=""),sep=",")
  write.table(res$exports,file=paste(outdir,"exports.csv",sep=""),sep=",")
  write.table(res$living,file=paste(outdir,"living.csv",sep=""),sep=",")
  write.table(res$changeR,file=paste(outdir,"changeRates.csv",sep=""),sep=",")
  write.table(res$dietcomposition,file=paste(outdir,"dietComposition.csv",sep=""),sep=",")
  write.table(res$bcf,file=paste(outdir,"bioconcentration.csv",sep=""),sep=",")
  write.table(res$bcf_l,file=paste(outdir,"bioconcentrationLipid.csv",sep=""),sep=",")
  write.table(res$baf,file=paste(outdir,"bioaccumulation.csv",sep=""),sep=",")
  write.table(res$baf_l,file=paste(outdir,"bioaccumulationLipid.csv",sep=""),sep=",")
  write.table(res$bmf,file=paste(outdir,"biomagnification.csv",sep=""),sep=",")
  write.table(res$bmf_l,file=paste(outdir,"biomagnificationLipid.csv",sep=""),sep=",")
  write.table(res$fishing,file=paste(outdir,"fishing.csv",sep=""),sep=",")
}


