#GENERATE LIM INPUT FILE FOR THE CONTAMINANT MODEL FROM TABULAR INPUTS
#Takes the directory where input files are located, and the estimated flow matrix and biomasses
#Returns the path to the generated LIM input file
createContaminantInputFile <- function(inputDir="gen_input", #directory where to take input files
                                        outPath="gen_input/auto_generated_contaminant1.input", #path of the autogenerated output LIM file
                                        logKOW=6, #logKOW is 6 for PCBs (see Toxlim and paper "Trophic Magnification of PCBs and Its Relationship to the Octanol-Water Partition Coefficient")
                                        KOC_KOW=0.41, #ratio of Koc over Kow (set to 0.41, see toxlim)
                                        OC=0.028, #organic carbon fraction of INST compartments, OC is the default 0.028 (see toxlim)
                                        flowMatrix=NULL,#estimated biomass flow matrix
                                        biomasses=NULL, #estimated biomasses
                                        pCH2_min=30,#minimum lipid layer permeation resistance, from Jan Hendriks et al., Environmental Toxicology and Chemistry, Vol. 20, No. 7, pp. 1399-1420, 2001 
                                        obj="minCont", #default objectives (minCont,maxCont,minUpdate,maxUpdate) that can be used with LINP
                                        relativeFlows=FALSE #sets whether biomass flows are divided by the biomass of the source node
                                        ){
  #avoid writing numbers in scientific notation
  options(scipen=.Machine$integer.max/2)
  normName <- function(name){
    return(gsub(" ","_",toupper(name)))
  }
  if(inputDir!="" && substr(inputDir,nchar(inputDir),nchar(inputDir)+1)!="/")
    inputDir<-paste(inputDir,"/",sep="")
  
  #read general input file
  input_params<-read.csv(file=paste(inputDir,"input_params.csv",sep=""))
  names<-as.vector(input_params$Group)
  for(i in 1:length(names))
    names[i]<-normName(names[i])
  input_params$Group<-names
  names1<-names
  names<-paste("C",names,sep="_")
  #get the names of detritus and primary producers
  detName<-names[input_params$Det==1]
  ppName<-names[input_params$PP==1]
  discardName<-names[input_params$Discard==1]
  if(length(discardName)>0)
    discardName<-discardName[1]
  #write stocks and exts
  fileOut<-outPath
  
  cat("#### COMPONENT", "\n", file=fileOut)
  for(i in 1:nrow(input_params))
    #if(!input_params$InstEq[i])
      cat(names[i], "\n", file=fileOut,append=TRUE)
  
  cat("C_WATER", "\n", file=fileOut,append=TRUE)
  
  cat("!Uptake rates", "\n", file=fileOut,append=TRUE)
  
  for(i in 1:nrow(input_params))
    if(!input_params$InstEq[i])
      cat(paste("W",names1[i],sep="_"), "\n", file=fileOut,append=TRUE)
  
  cat("\n#### END COMPONENT\n", file=fileOut,append=TRUE)
  
  paramString<-"\n### PARAMETERS\n!Biomass\n"
  for(i in 1:nrow(biomasses)){
    if(!is.na(biomasses[i]))
      paramString<-paste(paramString,"B_",rownames(biomasses)[i]," = ", biomasses[i],"\n",sep="")
  }
  paramString<-paste(paramString,"\n!PCB input data (ng/g)\n",sep="")
  
  for(i in 1:length(names)){
    if(!is.na(input_params$PCB[i]))
      paramString<-paste(paramString,"PCB_",names1[i]," = ", input_params$PCB[i],"\n",sep="")
    if(!is.na(input_params$PCBmin[i]))
      paramString<-paste(paramString,"PCB_",names1[i],"_MIN = ", input_params$PCBmin[i],"\n",sep="")
    if(!is.na(input_params$PCBmax[i]))
      paramString<-paste(paramString,"PCB_",names1[i],"_MAX = ", input_params$PCBmax[i],"\n",sep="")
  }
  paramString<-paste(paramString,"\n!Octanol/water partition ratio\nKOW = ",10^logKOW,"\n\n!ratio of Koc (organic carbon water partitioning coefficient) over Kow\nKOC_KOW = ",KOC_KOW,"\n\n!proportion of wet weight that is organic carbon in INST compartments\nOC = ",OC,sep="")
  
  paramString<-paste(paramString,"\n### END PARAMETERS\n",sep="")
  cat(paramString,file=fileOut,append=TRUE)

  
  totalCont<-"C_WATER"
  #totalCont<-""
  for(i in 1:length(names))
    totalCont<-paste(totalCont,names[i],sep=" + ")
  
  uptakeNams<-paste("W_",names1[!input_params$InstEq],sep="")
  totalUptake<-uptakeNams[1]
  for(i in 2:length(uptakeNams))
    totalUptake<-paste(totalUptake,uptakeNams[i],sep=" + ")
  
  if(!is.null(obj)&&obj=="minCont")
    cat("\n### COST", totalCont, "### END COST",file=fileOut,append=TRUE,sep="\n")
  if(!is.null(obj)&&obj=="maxCont")
    cat("\n### PROFIT", totalCont, "### END PROFIT",file=fileOut,append=TRUE,sep="\n")
  if(!is.null(obj)&&obj=="minUptake")
    cat("\n### COST", totalUptake, "### END COST",file=fileOut,append=TRUE,sep="\n")
  if(!is.null(obj)&&obj=="maxUptake")
    cat("\n### PROFIT", totalUptake, "### END PROFIT",file=fileOut,append=TRUE,sep="\n")
  
  #write equalities and inequalities
  eqString<-"\n### EQUATIONS\n!PCB data\n"
  ineqString<-"\n### INEQUALITIES\n!PCB data\n"
  
  eqCounter<-1
  pcb_data_eq<-c()
  for(i in 1:length(names)){
    if(!is.na(input_params$PCB[i])){
      eqString<-paste(eqString,names[i]," = ","PCB_",names1[i], "\n",sep="")
      pcb_data_eq<-c(pcb_data_eq,eqCounter)
      eqCounter<-eqCounter+1
    }
    if(!is.na(input_params$PCBmin[i]))
      ineqString<-paste(ineqString,names[i]," > ","PCB_",names1[i], "_MIN\n",sep="")
    if(!is.na(input_params$PCBmax[i]))
      ineqString<-paste(ineqString,names[i]," < ","PCB_",names1[i], "_MAX\n",sep="")
  }
  eqString<-paste(eqString,"\n!mass balances\n",sep="")
  kine_eq<-c()
  inst_eq<-c()

  for(i in 1:length(names)){
    if(!input_params$InstEq[i]){
    #eqString<-paste(eqString,"C_WATER*","W_",names1[i],sep="")
    #eqString<-paste(eqString,"B_",names1[i],"*W_",names1[i]," + ",(flowMatrix["IMP",names1[i]]+flowMatrix["DISC",names1[i]])/biomasses[i],"*",names[i],sep="")
    if(relativeFlows)
      eqString<-paste(eqString,"W_",names1[i]," + ",flowMatrix["IMP",names1[i]]/biomasses[i],"*",names[i],sep="")
    else eqString<-paste(eqString,"W_",names1[i]," + ",flowMatrix["IMP",names1[i]],"*",names[i],sep="")

    for(j in 1:length(names)){
      if(flowMatrix[names1[j],names1[i]]>0)
        if(relativeFlows)
          eqString<-paste(eqString," + ",flowMatrix[names1[j],names1[i]]/biomasses[j],"*",names[j],sep="")
        else eqString<-paste(eqString," + ",flowMatrix[names1[j],names1[i]],"*",names[j],sep="")
    }
    if(relativeFlows)
      eqString<-paste(eqString," = ",sum(flowMatrix[names1[i],])/biomasses[i],"*",names[i], "\n",sep="")
    else eqString<-paste(eqString," = ",sum(flowMatrix[names1[i],]),"*",names[i], "\n",sep="")
    
    kine_eq<-c(kine_eq,eqCounter)
    eqCounter<-eqCounter+1
  }
    else{
      eqString<-paste(eqString,names[i]," = C_WATER*OC*KOW*KOC_KOW\n",sep="")
      inst_eq<-c(inst_eq,eqCounter)
      eqCounter<-eqCounter+1
    }
  }
  
  uptakeWater_eq<-c()
  eqString<-paste(eqString,"!Uptake from water equality\n",sep="")
  for(i in 1:length(names1)){
    if(input_params$NoUptakeWater[i]){
      eqString<-paste(eqString,"W_",names1[i]," = 0\n",sep="")
      uptakeWater_eq<-c(uptakeWater_eq,eqCounter)
      eqCounter<-eqCounter+1
    }   
  }  
  
  ineqString<-paste(ineqString,"\n!non-negativity\n",sep="")
  for(i in 1:length(names)){
    if(is.na(input_params$PCB[i])&&is.na(input_params$PCBmin[i])&&is.na(input_params$PCBmax[i]))
      ineqString<-paste(ineqString,names[i]," > 0", "\n",sep="")
  }
  for(i in 1:length(names))
    if(!input_params$InstEq[i])
      ineqString<-paste(ineqString,"W_",names1[i]," > 0", "\n",sep="")
  
  #ineqString<-paste(ineqString,"C_WATER > 0", "\n",sep="")
  ineqString<-paste(ineqString,"C_WATER > 0.0001", "\n",sep="")
  
  #sets upper-bound for uptake from water: 1/(rho_CH2_min/KOW) (relative) or biomass/(rho_CH2_min/KOW)
  ineqString<-paste(ineqString,"!upper-bound on uptake rates from water\n",sep="")
  for(i in 1:length(names))
    if(!input_params$InstEq[i] && input_params$UptakeWaterUB[i])
      if(relativeFlows)
        ineqString<-paste(ineqString,"W_",names1[i]," < ", 1/(pCH2_min/10^logKOW),"*C_WATER\n",sep="")
      else ineqString<-paste(ineqString,"W_",names1[i]," < ", biomasses[i]/(pCH2_min/10^logKOW),"*C_WATER\n",sep="")

  eqString<-paste(eqString,"\n### END EQUATIONS\n",sep="")
  ineqString<-paste(ineqString,"\n### END INEQUALITIES\n",sep="")
  cat(eqString,ineqString,file=fileOut,append=TRUE,sep="\n")
  options(scipen=0)
  return(list(genFile=fileOut,inputTable=input_params,pcb_data_eq_indices=pcb_data_eq,
              inst_eq_indices=inst_eq,kine_eq_indices=kine_eq,uptakeWater_eq=uptakeWater_eq,relativeFlows=relativeFlows))
}

#GENERATE LIM INPUT FILE FOR THE BIOMASS MODEL FROM TABULAR INPUTS
#Takes the directory where input files are located, and returns the path to the generated LIM input file
createBiomassInputFile <- function(inputDir="gen_input", #directory where input files are located
                                   outPath="gen_input/auto_generated_trophic.input", #path of the autogenerated LIM output file
                                   respirationAssimilationConstr=TRUE,#if true add the constraint that R/A < 1
                                   obj=NULL #for the moment, either "minTST" or "maxTST" (minimize/maximize sum of flows). Then, it has to be solved with linear programming (Linp)
                                   ){
  #avoid writing numbers in scientific notation
  options(scipen=.Machine$integer.max/2)
  normName <- function(name){
    return(gsub(" ","_",toupper(name)))
  }
  if(inputDir!="" && substr(inputDir,nchar(inputDir),nchar(inputDir)+1)!="/")
    inputDir<-paste(inputDir,"/",sep="")
  
  #read general input file
  input_params<-read.csv(file=paste(inputDir,"input_params.csv",sep=""))
  names<-as.vector(input_params$Group)
  for(i in 1:length(names))
    names[i]<-normName(names[i])
  input_params$Group<-names
  #get the names of detritus, primary producers and fishing discards
  detName<-names[input_params$Det==1]
  ppName<-names[input_params$PP==1]
  discardName<-names[input_params$Discard==1]
  if(length(discardName)>1)
    warning(paste("More than one discard group specified. Considering only the first one: ",discardName[1],sep=""))
  if(length(discardName)>0)
    discardName<-discardName[1]
  else discardName<-"DISC"
  #write stocks and exts
  fileOut<-outPath
  
  cat("#### STOCK", "\n", file=fileOut)
  for(i in 1:nrow(input_params))
    cat(names[i], "\n", file=fileOut,append=TRUE)
  
  cat("#### END STOCK","","#### EXTERNAL","CO2","EXP","IMP","LAND",sep="\n", file=fileOut,append=TRUE)
  if(length(discardName)==0)
    cat("DISC",sep="\n", file=fileOut,append=TRUE)
  
  cat("#### END EXTERNAL",sep="\n", file=fileOut,append=TRUE)
  
  #read diet composition matrix (mean)
  diet_mean<-read.csv(file=paste(inputDir,"diet_mean.csv",sep=""),row.names=1)
  #read diet composition matrix (min)
  diet_min<-read.csv(file=paste(inputDir,"diet_min.csv",sep=""),row.names=1)
  #read diet composition matrix (max)
  diet_max<-read.csv(file=paste(inputDir,"diet_max.csv",sep=""),row.names=1)
  
  #write flows
  #flows to detritus (detritus and discards excluded)
  flowsString<-"\n### FLOWS\n!Flows to detritus\n"
  for(i in 1:length(names))
    for(j in 1:length(detName))
      if(!(names[i] %in% detName) && !(names[i] == discardName))
        flowsString<-paste(flowsString,names[i]," -> ",detName[j],"\n",sep="")
  
  #flows to CO2 (detritus and discards excluded)
  flowsString<-paste(flowsString,"\n!Flows to CO2\n",sep="")
  for(i in 1:length(names))
    if(!(names[i] %in% detName) && !(names[i] == discardName))
      flowsString<-paste(flowsString,names[i]," -> CO2\n",sep="")
  
  #flows from general imports
  flowsString<-paste(flowsString,"\n!Flows from imports\n",sep="")
  for(i in 1:length(names))
    flowsString<-paste(flowsString, "IMP -> ",names[i], "\n", sep="")
  
  #flows TO general exports
  flowsString<-paste(flowsString,"\n!Flows to exports\n",sep="")
  for(i in 1:length(names))
    flowsString<-paste(flowsString,names[i]," -> EXP\n",sep="")
  
  #landings
  flowsString<-paste(flowsString,"!Fishing = Landings + Discards\n!Landing flows\n",sep="")
  for(i in 1:length(names))
    if(!is.na(input_params$Lan[i]) || !is.na(input_params$Lanmin[i]) || !is.na(input_params$Lanmax[i]))
      flowsString<-paste(flowsString,names[i]," -> LAND\n",sep="")
  
  #discards
  flowsString<-paste(flowsString,"\n!Discard flows\n",sep="")
  for(i in 1:length(names))
    if(!is.na(input_params$Dis[i]) || !is.na(input_params$Dismin[i]) || !is.na(input_params$Dismax[i]))
      flowsString<-paste(flowsString,names[i]," -> ",discardName,"\n",sep="")
   
  flowsString<-paste(flowsString,"\n!Predation flows\n",sep="")
  for(j in 1:length(names))
    for(i in 1:length(names))
      if(!is.na(diet_mean[i,j]) || !is.na(diet_min[i,j]) || !is.na(diet_max[i,j]))
        flowsString<-paste(flowsString,names[i]," -> ",names[j],"\n",sep="")
  
  flowsString<-paste(flowsString,"\n### END FLOWS\n",sep="")
  cat(flowsString,file=fileOut,append=TRUE)
  
  paramString<-"\n### PARAMETERS\n!Biomass\n"
  for(i in 1:length(names)){
    if(!is.na(input_params$B[i]))
      paramString<-paste(paramString,"B_",names[i]," = ", input_params$B[i],"\n",sep="")
    
    if(!is.na(input_params$Bmin[i]))
      paramString<-paste(paramString,"BMIN_",names[i]," = ", input_params$Bmin[i],"\n",sep="")
    
    if(!is.na(input_params$Bmax[i]))
      paramString<-paste(paramString,"BMAX_",names[i]," = ", input_params$Bmax[i],"\n",sep="")
  }
  paramString<-paste(paramString,"\n!Consumption rate (in Ecopath Q/B)\n",sep="")
  for(i in 1:length(names)){
    if(!is.na(input_params$C[i]))
      paramString<-paste(paramString,"C_",names[i]," = ", input_params$C[i],"\n",sep="")
    
    if(!is.na(input_params$Cmin[i]))
      paramString<-paste(paramString,"CMIN_",names[i]," = ", input_params$Cmin[i],"\n",sep="")
    
    if(!is.na(input_params$Cmax[i]))
      paramString<-paste(paramString,"CMAX_",names[i]," = ", input_params$Cmax[i],"\n",sep="")
  }
  
  paramString<-paste(paramString,"\n!Proportional loss (in Ecopath 1-P/Q)\n",sep="")
  for(i in 1:length(names)){
    if(!is.na(input_params$C[i]) && !is.na(input_params$P[i]))
      paramString<-paste(paramString,"PL_",names[i]," = ", (1-input_params$P[i]/input_params$C[i]),"\n",sep="")
    
    if(!is.na(input_params$Cmin[i]) && !is.na(input_params$Pmax[i]))
      paramString<-paste(paramString,"PLMIN_",names[i]," = ", (1-input_params$Pmax[i]/input_params$Cmin[i]),"\n",sep="")
    else if(!is.na(input_params$Cmin[i]) && !is.na(input_params$P[i]))
      paramString<-paste(paramString,"PLMIN_",names[i]," = ", (1-input_params$P[i]/input_params$Cmin[i]),"\n",sep="")
    else if(!is.na(input_params$C[i]) && !is.na(input_params$Pmax[i]))
      paramString<-paste(paramString,"PLMIN_",names[i]," = ", (1-input_params$Pmax[i]/input_params$C[i]),"\n",sep="")
    
    if(!is.na(input_params$Cmax[i]) && !is.na(input_params$Pmin[i]))
      paramString<-paste(paramString,"PLMIN_",names[i]," = ", (1-input_params$Pmin[i]/input_params$Cmax[i]),"\n",sep="")
    else if(!is.na(input_params$Cmax[i]) && !is.na(input_params$P[i]))
      paramString<-paste(paramString,"PLMIN_",names[i]," = ", (1-input_params$P[i]/input_params$Cmax[i]),"\n",sep="")
    else if(!is.na(input_params$C[i]) && !is.na(input_params$Pmin[i]))
      paramString<-paste(paramString,"PLMIN_",names[i]," = ", (1-input_params$Pmin[i]/input_params$C[i]),"\n",sep="")
  }
  
  paramString<-paste(paramString,"\n### END PARAMETERS\n",sep="")
  cat(paramString,file=fileOut,append=TRUE)
  
  varString<-"\n### VARIABLES\n!Ingestion\n"
  #pp, detritus and discard excluded
  for(i in 1:length(names))
    if(!names[i] %in% detName && !names[i] %in% ppName && names[i] != discardName){
      varString<-paste(varString,"I_",names[i]," = FlowTo(",names[i], ") - IMP -> ",names[i], "\n", sep="")
    }
  
  varString<-paste(varString,"\n!UnassimilatedFood (respiration+detritus)\n",sep="")
  for(i in 1:length(names))
    if(!names[i] %in% detName && !names[i] %in% ppName && names[i] != discardName){
      varString<-paste(varString,"R_",names[i]," = ",names[i], " -> CO2", sep="")
      for(j in 1:length(detName))
        varString<-paste(varString," + ",names[i], " -> ",detName[j], sep="")
      varString<-paste(varString,"\n", sep="")
    }
  varString<-paste(varString,"\n### END VARIABLES\n",sep="")
  cat(varString,file=fileOut,append=TRUE)
  
  if(!is.null(obj)){
  tst<-paste("FlowFrom(",names[1],")",sep="")
  for(i in 2:length(names))
    tst<-paste(tst," + FlowFrom(",names[i],")",sep="")
  if(obj=="minTST")
    cat("\n### COST", tst, "### END COST",file=fileOut,append=TRUE,sep="\n")
  if(obj=="maxTST")
    cat("\n### PROFIT", tst, "### END PROFIT",file=fileOut,append=TRUE,sep="\n")
  }
  #write equalities and inequalities
  eqString<-"\n### EQUATIONS\n!Total ingestion  = biomass*consumption rate\n"
  ineqString<-"\n### INEQUALITIES\n!Total ingestion  = biomass*consumption rate\n"
  
  eqCounter<-1
  ingestionEqs<-c()
  for(i in 1:length(names)){
    if(!is.na(input_params$C[i]) && !is.na(input_params$B[i])){
      eqString<-paste(eqString,"I_",names[i]," = B_", names[i]," * C_", names[i], "\n",sep="")
      ingestionEqs<-c(ingestionEqs,eqCounter)
      eqCounter<-eqCounter+1
    }
    
    if(!is.na(input_params$Cmin[i]) && !is.na(input_params$Bmin[i]))
      ineqString<-paste(ineqString,"I_",names[i]," >= BMIN_", names[i]," * CMIN_", names[i], "\n",sep="")
    else if(!is.na(input_params$Cmin[i]) && !is.na(input_params$B[i]))
      ineqString<-paste(ineqString,"I_",names[i]," >= B_", names[i]," * CMIN_", names[i], "\n",sep="")
    else if(!is.na(input_params$C[i]) && !is.na(input_params$Bmin[i]))
      ineqString<-paste(ineqString,"I_",names[i]," >= BMIN_", names[i]," * C_", names[i], "\n",sep="")
    
    if(!is.na(input_params$Cmax[i]) && !is.na(input_params$Bmax[i]))
      ineqString<-paste(ineqString,"I_",names[i]," <= BMAX_", names[i]," * CMAX_", names[i], "\n",sep="")
    else if(!is.na(input_params$Cmax[i]) && !is.na(input_params$B[i]))
      ineqString<-paste(ineqString,"I_",names[i]," <= B_", names[i]," * CMAX_", names[i], "\n",sep="")
    else if(!is.na(input_params$C[i]) && !is.na(input_params$Bmax[i]))
      ineqString<-paste(ineqString,"I_",names[i]," <= BMAX_", names[i]," * C_", names[i], "\n",sep="")
  }
  
  eqString<-paste(eqString,"\n!Unassimilated food\n",sep="")
  ineqString<-paste(ineqString,"\n!Unassimilated food\n",sep="")
  egestionEqs<-c()
  
  for(i in 1:length(names)){
    if(!is.na(input_params$C[i]) && !is.na(input_params$P[i])){
      eqString<-paste(eqString,"R_",names[i]," = I_", names[i]," * PL_", names[i], "\n",sep="")
      egestionEqs<-c(egestionEqs,eqCounter)
      eqCounter<-eqCounter+1
    }
    if((!is.na(input_params$Cmin[i]) && (!is.na(input_params$Pmax[i]) || !is.na(input_params$P[i])))|| (!is.na(input_params$C[i]) && !is.na(input_params$Pmax[i])) )
      ineqString<-paste(ineqString,"R_",names[i]," >= I_", names[i]," * PLMIN_", names[i], "\n",sep="")
    
    if((!is.na(input_params$Cmax[i]) && (!is.na(input_params$Pmin[i]) || !is.na(input_params$P[i])))|| (!is.na(input_params$C[i]) && !is.na(input_params$Pmin[i])) )
      ineqString<-paste(ineqString,"R_",names[i]," <= I_", names[i]," * PLMAX_", names[i], "\n",sep="")
    #R/A < 1
    if(respirationAssimilationConstr && !names[i] %in% detName && !names[i] %in% ppName && names[i] != discardName)
      ineqString<-paste(ineqString,names[i], " -> CO2 <= I_", names[i]," - R_", names[i], "\n",sep="")
  }
  
  
  eqString<-paste(eqString,"\n!Diet considerations\n",sep="")
  ineqString<-paste(ineqString,"\n!Diet considerations\n",sep="")
  dietEqs<-c()
  
  for(j in 1:length(names)){
    #handle the case where the import represent part of the diet
    if(!is.na(input_params$DietFromImport[j])){
      eqString<-paste(eqString,"!import considered part of the diet\nIMP -> ",names[j]," = I_",names[j]," * ", input_params$DietFromImport[j], "\n",sep="")
      dietEqs<-c(dietEqs,eqCounter)
      eqCounter<-eqCounter+1
    }
    
    if(!is.na(input_params$DietFromImportMin[j]))
      ineqString<-paste(ineqString,names[i],"!import considered part of the diet\nIMP -> ",names[j]," >= I_",names[j]," * ", input_params$DietFromImportMin[j], "\n",sep="")
    
    if(!is.na(input_params$DietFromImportMax[j]))
      ineqString<-paste(ineqString,names[i],"!import considered part of the diet\nIMP -> ",names[j]," <= I_",names[j]," * ", input_params$DietFromImportMax[j], "\n",sep="")
    
    for(i in 1:length(names)){
      if(!is.na(diet_mean[i,j])){
        eqString<-paste(eqString,names[i]," -> ",names[j]," = I_",names[j]," * ", diet_mean[i,j], "\n",sep="")
        dietEqs<-c(dietEqs,eqCounter)
        eqCounter<-eqCounter+1
      }
        
      if(!is.na(diet_min[i,j]))
        ineqString<-paste(ineqString,names[i]," -> ",names[j]," >= I_",names[j]," * ", diet_min[i,j], "\n",sep="")
      if(!is.na(diet_max[i,j]))
        ineqString<-paste(ineqString,names[i]," -> ",names[j]," <= I_",names[j]," * ", diet_max[i,j], "\n",sep="")
     }
    }
  
  #landings
  eqString<-paste(eqString,"\n!Landings\n",sep="")
  ineqString<-paste(ineqString,"\n!Landings\n",sep="")
  landDisc_eq<-c()
  for(i in 1:length(names)){
    if(!is.na(input_params$Lan[i])){
      eqString<-paste(eqString,names[i]," -> LAND = ",input_params$Lan[i],"\n",sep="")
      landDisc_eq<-c(landDisc_eq,eqCounter)
      eqCounter<-eqCounter+1
    }
    if(!is.na(input_params$Lanmin[i]))
      ineqString<-paste(ineqString,names[i]," -> LAND >= ",input_params$Lanmin[i],"\n",sep="")
    
    if(!is.na(input_params$Lanmax[i]))
      ineqString<-paste(ineqString,names[i]," -> LAND <= ",input_params$Lanmax[i],"\n",sep="")
  }
  #discards
  eqString<-paste(eqString,"\n!Discards\n",sep="")
  ineqString<-paste(ineqString,"\n!Discards\n",sep="")
  
  for(i in 1:length(names)){
    if(!is.na(input_params$Dis[i])){
      eqString<-paste(eqString,names[i]," -> ",discardName," = ",input_params$Dis[i],"\n",sep="")
      landDisc_eq<-c(landDisc_eq,eqCounter)
      eqCounter<-eqCounter+1
    }
    if(!is.na(input_params$Dismin[i]))
      ineqString<-paste(ineqString,names[i]," -> ",discardName," >= ",input_params$Dismin[i],"\n",sep="")
    if(!is.na(input_params$Dismax[i]))
      ineqString<-paste(ineqString,names[i]," -> ",discardName," <= ",input_params$Dismax[i],"\n",sep="")
  }
  
  eqString<-paste(eqString,"\n### END EQUATIONS\n",sep="")
  ineqString<-paste(ineqString,"\n### END INEQUALITIES\n",sep="")
  cat(eqString,ineqString,file=fileOut,append=TRUE,sep="\n")
  options(scipen=0)
  return(list(genFile=fileOut,inputTable=input_params,
              ingestion_eq_indices=ingestionEqs,egestion_eq_indices=egestionEqs,
              diet_eq_indices=dietEqs,landDisc_eq_indices=landDisc_eq))
}
